love.graphics.setDefaultFilter("nearest", "nearest")

-- Loads everything
require("cde/load")

-- Misc. functions
require("cde/misc")

-- Floor related functions
require("cde/floor")

-- The majority of raycasting-related functions
require("cde/raycasting")

-- Spritecasting
require("cde/spritecasting")

-- Table Serialiser
require("cde/tableSer")

-- Shader
pers = require("cde/pers")

function love.load()
	loadEverything()
	
	TILT = 0
	tLim = 160
end

function love.update(dt)
	-- Key Input
	input(dt)

	for x = 0, w, 1 do
		castRay(x)
	end

	updateDoor(dt)
	castSprites()
	castDoors()

	if not curShrink then
		curSize = curSize + dt * 50

		if curSize > maxSize then
			curShrink = true
		end
	else
		curSize = curSize - dt * 50

		if curSize < minSize then
			curShrink = false
		end
	end
end

function love.draw()
	canvas:clear()

	drawFloor()

	love.graphics.setCanvas(canvas)
		
		-- Sort the spritePool based on the screen depth
		table.sort(spritePool, zSort)

		-- Walls draw before the sprites
		for i,v in ipairs(wallPool) do
			love.graphics.setColor(v.color)
			if img[v.texture] then
				love.graphics.draw(img[v.texture], quads[v.hor], v.x, v.start, 0, 1, v.scale)
			end
		end

		for i,v in ipairs(spritePool) do
			love.graphics.setColor(v.color)
			if v.type == "wall" then
				if img[v.texture] then
					love.graphics.draw(img[v.texture], quads[v.hor], v.x, v.start, 0, 1, v.scale)
				end
			elseif v.type == "sprite" then
				love.graphics.draw(spriteImg[v.id], quads[v.hor], v.x, v.y, 0, 1, v.scale)
			elseif v.type == "door" then
				beginSt = v.cov[1]
				endSt = v.cov[2]

				love.graphics.setStencil( stencil )
					pers.quad(img[v.id], v.v1, v.v2, v.v3, v.v4)
					--love.graphics.polygon("fill", {v.v1[1], v.v1[2], v.v2[1], v.v2[2],v.v3[1], v.v3[2],v.v4[1], v.v4[2],})
				love.graphics.setStencil()
			end
		end
		
		-- Empty the pools
		spritePool = {}
		wallPool = {}
		floorTable = {}
	love.graphics.setCanvas()

	love.graphics.setColor(255, 255, 255)
	love.graphics.draw(canvas, 0, 0)

	love.graphics.setColor(255, 20, 20)
	love.graphics.print(string.format("FPS: %1.f\nPlayer Rotation: %1.f\nX: %.1f\nY: %.1f", love.timer.getFPS(), math.deg(getPlayerRot()), player.posX, player.posY))
end

function love.keypressed(key)
	if key == "escape" then love.event.quit() end
	if key == "tab" then grabbed = not grabbed end
end